@@ -6,7 +6,7 @@
 
 use flv_util::string_helper::upper_cammel_case_to_sentence;
 use fluvio_protocol::{Encoder, Decoder};
-use crate::smartstream::SmartStreamRuntimeError;
+use crate::smartmodule::SmartModuleRuntimeError;
 
 // -----------------------------------
 // Error Definition & Implementation
@@ -14,6 +14,7 @@ use crate::smartstream::SmartStreamRuntimeError;
 
 #[repr(i16)]
 #[derive(thiserror::Error, Encoder, Decoder, PartialEq, Debug, Clone)]
+#[non_exhaustive]
 pub enum ErrorCode {
     #[fluvio(tag = -1)]
     #[error("An unknown server error occurred")]
@@ -24,6 +25,10 @@ pub enum ErrorCode {
     #[error("ErrorCode indicated success. If you see this it is likely a bug.")]
     None,
 
+    #[fluvio(tag = 2)]
+    #[error("Other error: {0}")]
+    Other(String),
+
     #[fluvio(tag = 1)]
     #[error("Offset out of range")]
     OffsetOutOfRange,
@@ -39,6 +44,12 @@ pub enum ErrorCode {
     #[fluvio(tag = 56)]
     #[error("a storage error occurred")]
     StorageError,
+    #[fluvio(tag = 60)]
+    #[error("invalid create request")]
+    InvalidCreateRequest,
+    #[fluvio(tag = 61)]
+    #[error("invalid Delete request")]
+    InvalidDeleteRequest,
 
     // Spu errors
     #[fluvio(tag = 1000)]
@@ -93,34 +104,68 @@ pub enum ErrorCode {
     #[error("the fetch session was not found")]
     FetchSessionNotFoud,
 
-    // SmartStream errors
+    // Legacy SmartModule errors
+    #[deprecated(since = "0.9.13")]
     #[fluvio(tag = 4000)]
-    #[error("a SmartStream error occurred")]
-    SmartStreamError(#[from] SmartStreamError),
+    #[error("a legacy SmartModule error occurred")]
+    LegacySmartModuleError(#[from] LegacySmartModuleError),
 
     // Managed Connector Errors
     #[fluvio(tag = 5000)]
     #[error("an error occurred while managing a connector")]
     ManagedConnectorError,
+
     #[fluvio(tag = 5001)]
     #[error("the managed connector was not found")]
     ManagedConnectorNotFound,
 
-    // Smart Module Errors
+    #[fluvio(tag = 5002)]
+    #[error("an error occurred while managing a connector")]
+    ManagedConnectorAlreadyExists,
+
+    // SmartModule Errors
     #[fluvio(tag = 6000)]
-    #[error("an error occurred while managing a smart module")]
+    #[error("an error occurred while managing a SmartModule")]
     SmartModuleError,
     #[fluvio(tag = 6001)]
-    #[error("the smart module was not found")]
-    SmartModuleNotFound,
+    #[error("SmartModule {name} was not found")]
+    SmartModuleNotFound { name: String },
+    #[fluvio(tag = 6002)]
+    #[error("SmartModule is invalid")]
+    SmartModuleInvalid { error: String, name: Option<String> },
+    #[fluvio(tag = 6003)]
+    #[error("SmartModule is not a valid '{kind}' SmartModule due to {error}. Are you missing a #[smartmodule({kind})] attribute?")]
+    SmartModuleInvalidExports { error: String, kind: String },
+    #[fluvio(tag = 6004)]
+    #[error("SmartModule runtime error {0}")]
+    SmartModuleRuntimeError(SmartModuleRuntimeError),
 
-    // Table Errors
+    // TableFormat Errors
     #[fluvio(tag = 7000)]
-    #[error("a table error occurred")]
-    TableError,
+    #[error("a tableformat error occurred")]
+    TableFormatError,
     #[fluvio(tag = 7001)]
-    #[error("the table was not found")]
-    TableNotFound,
+    #[error("the tableformat was not found")]
+    TableFormatNotFound,
+    #[fluvio(tag = 7002)]
+    #[error("the tableformat already exists")]
+    TableFormatAlreadyExists,
+
+    // DerivedStream Object Errors
+    #[fluvio(tag = 8000)]
+    #[error("DerivedStream object error")]
+    DerivedStreamObjectError,
+    #[fluvio(tag = 8001)]
+    #[error("the derivedstream was not found")]
+    DerivedStreamNotFound(String),
+    #[fluvio(tag = 8002)]
+    #[error("the derivedstream join data cannot be fetched")]
+    DerivedStreamJoinFetchError,
+    #[fluvio(tag = 8003)]
+    #[error("the derivedstream {0} is invalid")]
+    DerivedStreamInvalid(String),
+    #[error("can't do recursive derivedstream yet: {0}->{1}")]
+    DerivedStreamRecursion(String, String),
 }
 
 impl Default for ErrorCode {
@@ -146,20 +191,18 @@ impl ErrorCode {
     }
 }
 
-/// A type representing the possible errors that may occur during SmartStream execution.
-// This is also where we can update our error representation in the future
-// TODO: Add variant for reporting panics
+/// Deprecated. A type representing the possible errors that may occur during DerivedStream execution.
 #[derive(thiserror::Error, Debug, Clone, PartialEq, Encoder, Decoder)]
-pub enum SmartStreamError {
+pub enum LegacySmartModuleError {
     #[error("Runtime error")]
-    Runtime(#[from] SmartStreamRuntimeError),
+    Runtime(#[from] SmartModuleRuntimeError),
     #[error("WASM Module error: {0}")]
     InvalidWasmModule(String),
-    #[error("WASM module is not a valid '{0}' SmartStream. Are you missing a #[smartstream({0})] attribute?")]
-    InvalidSmartStreamModule(String),
+    #[error("WASM module is not a valid '{0}' DerivedStream. Are you missing a #[smartmodule({0})] attribute?")]
+    InvalidDerivedStreamModule(String),
 }
 
-impl Default for SmartStreamError {
+impl Default for LegacySmartModuleError {
     fn default() -> Self {
         Self::Runtime(Default::default())
     }
